@@ -1,13 +1,15 @@
 import copy
 import logging
-import numpy as np
 import string
+import warnings
+from typing import Callable, List, Optional, Tuple, Union, cast
+
+import numpy as np
 import tensorflow as tf
 
 from alibi.api.defaults import DEFAULT_DATA_INTGRAD, DEFAULT_META_INTGRAD
-from alibi.utils.approximation_methods import approximation_parameters
 from alibi.api.interfaces import Explainer, Explanation
-from typing import Callable, Union, List, Tuple, Optional
+from alibi.utils.approximation_methods import approximation_parameters
 
 logger = logging.getLogger(__name__)
 
@@ -29,7 +31,7 @@ def _compute_convergence_delta(model: Union[tf.keras.models.Model],
     Parameters
     ----------
     model
-        Tensorflow or keras model.
+        `tensorflow` model.
     input_dtypes
         List with data types of the inputs.
     attributions
@@ -43,18 +45,17 @@ def _compute_convergence_delta(model: Union[tf.keras.models.Model],
     target
         Target for which the gradients are calculated for classification models.
     _is_list
-        Whether the model's input is a list (multiple inputs) or a np array (single input).
+        Whether the model's input is a `list` (multiple inputs) or a `np.narray` (single input).
 
     Returns
     -------
-        Convergence deltas for each data point.
+    Convergence deltas for each data point.
     """
     if forward_kwargs is None:
         forward_kwargs = {}
     if _is_list:
         start_point = [tf.convert_to_tensor(start_point[k], dtype=input_dtypes[k]) for k in range(len(input_dtypes))]
         end_point = [tf.convert_to_tensor(end_point[k], dtype=input_dtypes[k]) for k in range(len(input_dtypes))]
-
     else:
         start_point = tf.convert_to_tensor(start_point)
         end_point = tf.convert_to_tensor(end_point)
@@ -97,7 +98,7 @@ def _compute_convergence_delta(model: Union[tf.keras.models.Model],
 def _select_target(preds: tf.Tensor,
                    targets: Union[None, tf.Tensor, np.ndarray, list]) -> tf.Tensor:
     """
-    Select the predictions corresponding to the targets if targets is not None.
+    Select the predictions corresponding to the targets if targets is not ``None``.
 
     Parameters
     ----------
@@ -105,9 +106,10 @@ def _select_target(preds: tf.Tensor,
         Predictions before selection.
     targets
         Targets to select.
+
     Returns
     -------
-        Selected predictions
+    Selected predictions.
 
     """
     if targets is not None:
@@ -121,25 +123,27 @@ def _select_target(preds: tf.Tensor,
 
 
 def _run_forward(model: Union[tf.keras.models.Model],
-                 x: Union[List[tf.Tensor], List[np.ndarray]],
+                 x: Union[List[tf.Tensor], List[np.ndarray], tf.Tensor, np.ndarray],
                  target: Union[None, tf.Tensor, np.ndarray, list],
                  forward_kwargs: Optional[dict] = None) -> tf.Tensor:
     """
-    Returns the output of the model. If the target is not `None`, only the output for the selected target is returned.
+    Returns the output of the model. If the target is not ``None``, only the output for the selected
+    target is returned.
 
     Parameters
     ----------
     model
-        Tensorflow or keras model.
+        `tensorflow` model.
     x
         Input data point.
     target
         Target for which the gradients are calculated for classification models.
     forward_kwargs
         Input keyword args.
+
     Returns
     -------
-        Model output or model output after target selection for classification models.
+    Model output or model output after target selection for classification models.
 
     """
     if forward_kwargs is None:
@@ -168,37 +172,39 @@ def _run_forward_from_layer(model: tf.keras.models.Model,
     Parameters
     ----------
     model
-        Tensorflow or keras model.
+        `tensorflow` model.
     layer
         Starting layer for the forward call.
     orig_call
         Original `call` method of the layer.
     orig_dummy_input
         Dummy input needed to initiate the model forward call. The number of instances in the dummy input must
-        be the same as the number of instances in x. The dummy input values play no role in the evaluation
+        be the same as the number of instances in `x`. The dummy input values play no role in the evaluation
         as the  layer's status is overwritten during the forward call.
     x
         Layer's inputs. The layer's status is overwritten with `x` during the forward call.
     target
         Target for the output position to be returned.
     forward_kwargs
-        Input keyword args. It must be a dict with numpy arrays as values. If it's not None,
-        the first dimension of the arrays must correspond to the number of instances in x and orig_dummy_input.
+        Input keyword args. It must be a dict with `numpy` arrays as values. If it's not ``None``,
+        the first dimension of the arrays must correspond to the number of instances in `x` and orig_dummy_input.
     run_from_layer_inputs
-        If True, the forward pass starts from the layer's inputs, if False it starts from the layer's outputs.
+        If ``True``, the forward pass starts from the layer's inputs, if ``False`` it starts from the layer's outputs.
     select_target
         Whether to return predictions for selected targets or return predictions for all targets.
 
     Returns
     -------
-        Model's predictions for the given target.
+    Model's predictions for the given target.
 
     """
+
     def feed_layer(layer):
         """
         Overwrites the intermediate layer status with the precomputed values `x`.
 
         """
+
         def decorator(func):
             def wrapper(*args, **kwargs):
                 # Store the result and inputs of `layer.call` internally.
@@ -242,7 +248,7 @@ def _run_forward_to_layer(model: tf.keras.models.Model,
     Parameters
     ----------
     model
-        Tensorflow or keras model.
+        `tensorflow` model.
     layer
         Starting layer for the forward call.
     orig_call
@@ -252,11 +258,11 @@ def _run_forward_to_layer(model: tf.keras.models.Model,
     forward_kwargs
         Input keyword args.
     run_to_layer_inputs
-        If True, the layer's inputs are returned. If False, the layer's output's are returned.
+        If ``True``, the layer's inputs are returned. If ``False``, the layer's output's are returned.
 
     Returns
     -------
-        Output of the given layer.
+    Output of the given layer.
 
     """
     if forward_kwargs is None:
@@ -267,6 +273,7 @@ def _run_forward_to_layer(model: tf.keras.models.Model,
         Stores the layer's outputs internally to the layer's object.
 
         """
+
         def decorator(func):
             def wrapper(*args, **kwargs):
                 # Store the result of `layer.call` internally.
@@ -312,7 +319,7 @@ def _forward_input_baseline(X: Union[List[np.ndarray], np.ndarray],
     bls
         Baselines.
     model
-        Tensorflow or keras model.
+        `tensorflow` model.
     layer
         Desired layer output.
     orig_call
@@ -320,11 +327,12 @@ def _forward_input_baseline(X: Union[List[np.ndarray], np.ndarray],
     forward_kwargs
         Input keyword args.
     forward_to_inputs
-        If True, X and bls are forwarded to the layer's input. If False, they are forwarded to the layer's outputs.
+        If ``True``, `X` and bls are forwarded to the layer's input. If ``False``, they are forwarded to
+        the layer's outputs.
 
     Returns
     -------
-        Forwarded inputs and  baselines as a numpy arrays.
+    Forwarded inputs and baselines as a `numpy` arrays.
 
     """
     if forward_kwargs is None:
@@ -366,16 +374,17 @@ def _gradients_input(model: Union[tf.keras.models.Model],
     Parameters
     ----------
     model
-        Tensorflow or keras model.
+        `tensorflow` model.
     x
         Input data point.
     target
         Target for which the gradients are calculated if the output dimension is higher than 1.
     forward_kwargs
         Input keyword args.
+
     Returns
     -------
-        Gradients for each input feature.
+    Gradients for each input feature.
 
     """
     if forward_kwargs is None:
@@ -404,12 +413,12 @@ def _gradients_layer(model: Union[tf.keras.models.Model],
     Parameters
     ----------
     model
-        Tensorflow or keras model.
+        `tensorflow` model.
     layer
         Layer of the model with respect to which the gradients are calculated.
     orig_call
         Original `call` method of the layer. This is necessary since the call method is modified by the function
-        in order to make the layer output visible to the GradientTape.
+        in order to make the layer output visible to the `GradientTape`.
     x
         Input data point.
     target
@@ -417,14 +426,15 @@ def _gradients_layer(model: Union[tf.keras.models.Model],
     forward_kwargs
         Input keyword args.
     compute_layer_inputs_gradients
-        If True, gradients are computed with respect to the layer's inputs.
-        If False, they are computed with respect to the layer's outputs.
+        If ``True``, gradients are computed with respect to the layer's inputs.
+        If ``False``, they are computed with respect to the layer's outputs.
 
     Returns
     -------
-        Gradients for each element of layer.
+    Gradients for each element of layer.
 
     """
+
     def watch_layer(layer, tape):
         """
         Make an intermediate hidden `layer` watchable by the `tape`.
@@ -493,7 +503,7 @@ def _gradients_layer(model: Union[tf.keras.models.Model],
 def _format_baseline(X: np.ndarray,
                      baselines: Union[None, int, float, np.ndarray]) -> np.ndarray:
     """
-    Formats baselines to return a numpy array.
+    Formats baselines to return a `numpy` array.
 
     Parameters
     ----------
@@ -504,7 +514,7 @@ def _format_baseline(X: np.ndarray,
 
     Returns
     -------
-        Formatted inputs and  baselines as a numpy arrays.
+    Formatted inputs and  baselines as a `numpy` arrays.
 
     """
     if baselines is None:
@@ -533,7 +543,7 @@ def _format_target(target: Union[None, int, list, np.ndarray],
 
     Returns
     -------
-        Formatted target as a list.
+    Formatted target as a list.
 
     """
     if target is not None:
@@ -547,6 +557,50 @@ def _format_target(target: Union[None, int, list, np.ndarray],
     return target
 
 
+def _get_target_from_target_fn(target_fn: Callable,
+                               model: tf.keras.Model,
+                               X: Union[np.ndarray, List[np.ndarray]],
+                               forward_kwargs: Optional[dict] = None) -> np.ndarray:
+    """
+    Generate a target vector by using the `target_fn` to pick out a
+    scalar dimension from the predictions.
+
+    Parameters
+    ----------
+    target_fn
+        Target function.
+    model
+        Model.
+    X
+        Data to be explained.
+    forward_kwargs
+        Any additional kwargs needed for the model forward pass.
+
+    Returns
+    -------
+    Integer array of dimension `(N, )`.
+    """
+    if forward_kwargs is None:
+        preds = model(X)
+    else:
+        preds = model(X, **forward_kwargs)
+
+    # raise a warning if the predictions are scalar valued already
+    # TODO: in the future we want to support outputs that are >2D at which point this check should change
+    if preds.shape[-1] == 1:
+        msg = "Predictions from the model are scalar valued but `target_fn` was passed. `target_fn` is not necessary" \
+              "when predictions are scalar valued already. Using `target_fn` here may result in unexpected behaviour."
+        warnings.warn(msg)
+
+    target = target_fn(preds)
+    expected_shape = (target.shape[0],)
+    if target.shape != expected_shape:
+        # TODO: in the future we want to support outputs that are >2D at which point this check should change
+        msg = f"`target_fn` returned an array of shape {target.shape} but expected an array of shape {expected_shape}."
+        raise ValueError(msg)  # TODO: raise a more specific error type?
+    return target
+
+
 def _sum_integral_terms(step_sizes: list,
                         grads: Union[tf.Tensor, np.ndarray]) -> Union[tf.Tensor, np.ndarray]:
     """
@@ -561,7 +615,7 @@ def _sum_integral_terms(step_sizes: list,
 
     Returns
     -------
-        Sums of the gradients along the chosen path.
+    Sums of the gradients along the chosen path.
 
     """
     input_str = string.ascii_lowercase[1: len(grads.shape)]
@@ -593,7 +647,7 @@ def _calculate_sum_int(batches: List[List[tf.Tensor]],
     batches
         List of batch gradients.
     model
-        tf.keras or keras model.
+        `tf.keras` or `keras` model.
     target
         List of targets.
     target_paths
@@ -609,7 +663,7 @@ def _calculate_sum_int(batches: List[List[tf.Tensor]],
 
     Returns
     -------
-
+    Sums of the gradients along the chosen path.
     """
     grads = tf.concat(batches[j], 0)
     shape = grads.shape[1:]
@@ -636,13 +690,9 @@ def _validate_output(model: tf.keras.Model,
     Parameters
     ----------
     model
-        Keras model for which the output is validated.
+        `Keras` model for which the output is validated.
     target
         Targets for which gradients are calculated
-
-    Returns
-    -------
-
     """
     if not model.output_shape or not any(isinstance(model.output_shape, t) for t in _valid_output_shape_type):
         raise NotImplementedError(f"The model output_shape attribute must be in {_valid_output_shape_type}. "
@@ -663,26 +713,26 @@ class IntegratedGradients(Explainer):
     def __init__(self,
                  model: tf.keras.Model,
                  layer: Optional[tf.keras.layers.Layer] = None,
+                 target_fn: Optional[Callable] = None,
                  method: str = "gausslegendre",
                  n_steps: int = 50,
                  internal_batch_size: int = 100
                  ) -> None:
         """
-        An implementation of the integrated gradients method for Tensorflow and Keras models.
+        An implementation of the integrated gradients method for `tensorflow` models.
 
-        For details of the method see the original paper:
-        https://arxiv.org/abs/1703.01365 .
+        For details of the method see the original paper: https://arxiv.org/abs/1703.01365 .
 
         Parameters
         ----------
         model
-            Tensorflow or Keras model.
+            `tensorflow` model.
         layer
             Layer with respect to which the gradients are calculated.
             If not provided, the gradients are calculated with respect to the input.
         method
             Method for the integral approximation. Methods available:
-            "riemann_left", "riemann_right", "riemann_middle", "riemann_trapezoid", "gausslegendre".
+            ``"riemann_left"``, ``"riemann_right"``, ``"riemann_middle"``, ``"riemann_trapezoid"``, ``"gausslegendre"``.
         n_steps
             Number of step in the path integral approximation from the baseline to the input instance.
         internal_batch_size
@@ -701,8 +751,8 @@ class IntegratedGradients(Explainer):
             self._has_inputs = True
 
         if layer is None:
-            self.orig_call = None
-            layer_num = 0
+            self.orig_call: Optional[Callable] = None
+            layer_num: Optional[int] = 0
         else:
             self.orig_call = layer.call
             try:
@@ -722,11 +772,13 @@ class IntegratedGradients(Explainer):
         self._is_np: Optional[bool] = None
         self.orig_dummy_input: Optional[Union[list, np.ndarray]] = None
 
+        self.target_fn = target_fn
+
     def explain(self,
                 X: Union[np.ndarray, List[np.ndarray]],
                 forward_kwargs: Optional[dict] = None,
-                baselines: Union[int, float, np.ndarray, List[int], List[float], List[np.ndarray]] = None,
-                target: Union[int, list, np.ndarray] = None,
+                baselines: Optional[Union[int, float, np.ndarray, List[int], List[float], List[np.ndarray]]] = None,
+                target: Optional[Union[int, list, np.ndarray]] = None,
                 attribute_to_layer_inputs: bool = False) -> Explanation:
         """Calculates the attributions for each input feature or element of layer and
         returns an Explanation object.
@@ -736,13 +788,13 @@ class IntegratedGradients(Explainer):
         X
             Instance for which integrated gradients attribution are computed.
         forward_kwargs
-            Input keyword args. If it's not None, it must be a dict with numpy arrays as values.
+            Input keyword args. If it's not ``None``, it must be a dict with `numpy` arrays as values.
             The first dimension of the arrays must correspond to the number of examples.
-            It will be repeated for each of n_steps along the integrated path.
+            It will be repeated for each of `n_steps` along the integrated path.
             The attributions are not computed with respect to these arguments.
         baselines
             Baselines (starting point of the path integral) for each instance.
-            If the passed value is an `np.ndarray` must have the same shape as X.
+            If the passed value is an `np.ndarray` must have the same shape as `X`.
             If not provided, all features values for the baselines are set to 0.
         target
             Defines which element of the model output is considered to compute the gradients.
@@ -753,26 +805,37 @@ class IntegratedGradients(Explainer):
             For classification models `target` can be either the true classes or the classes predicted by the model.
         attribute_to_layer_inputs
             In case of layers gradients, controls whether the gradients are computed for the layer's inputs or
-            outputs. If True, gradients are computed for the layer's inputs, if False for the layer's outputs.
+            outputs. If ``True``, gradients are computed for the layer's inputs, if ``False`` for the layer's outputs.
 
         Returns
         -------
+        explanation
             `Explanation` object including `meta` and `data` attributes with integrated gradients attributions
-            for each feature.
+            for each feature. See usage at `IG examples`_ for details.
 
+            .. _IG examples:
+                https://docs.seldon.io/projects/alibi/en/latest/methods/IntegratedGradients.html
         """
+        # target handling logic
+        if self.target_fn and target is not None:
+            msg = 'Both `target_fn` and `target` were provided. Only one of these should be provided.'
+            raise ValueError(msg)
+        if self.target_fn:
+            target = _get_target_from_target_fn(self.target_fn, self.model, X, forward_kwargs)
+
         self._is_list = isinstance(X, list)
         self._is_np = isinstance(X, np.ndarray)
         if forward_kwargs is None:
             forward_kwargs = {}
 
         if self._is_list:
+            X = cast(List[np.ndarray], X)  # help mypy out
             self.orig_dummy_input = [np.zeros((1,) + xx.shape[1:], dtype=xx.dtype) for xx in X]  # type: ignore
             nb_samples = len(X[0])
             input_dtypes = [xx.dtype for xx in X]
             # Formatting baselines in case of models with multiple inputs
             if baselines is None:
-                baselines = [None for _ in range(len(X))]
+                baselines = [None for _ in range(len(X))]  # type: ignore
             else:
                 if not isinstance(baselines, list):
                     raise ValueError(f"If the input X is a list, baseline can only be `None` or "
@@ -792,11 +855,12 @@ class IntegratedGradients(Explainer):
                 baselines[i] = baseline  # type: ignore
 
         elif self._is_np:
+            X = cast(np.ndarray, X)  # help mypy out
             self.orig_dummy_input = np.zeros((1,) + X.shape[1:], dtype=X.dtype)  # type: ignore
             nb_samples = len(X)
             input_dtypes = [X.dtype]  # type: ignore
             # Formatting baselines for models with a single input
-            baselines = _format_baseline(X, baselines)
+            baselines = _format_baseline(X, baselines)  # type: ignore # TODO: validate/narrow baselines type
 
         else:
             raise ValueError("Input must be a np.ndarray or a list of np.ndarray")
@@ -804,9 +868,10 @@ class IntegratedGradients(Explainer):
         # defining integral method
         step_sizes_func, alphas_func = approximation_parameters(self.method)
         step_sizes, alphas = step_sizes_func(self.n_steps), alphas_func(self.n_steps)
-        target = _format_target(target, nb_samples)
+        target = _format_target(target, nb_samples)  # type: ignore[assignment]
 
         if self._is_list:
+            X = cast(List[np.ndarray], X)  # help mypy out
             # Attributions calculation in case of multiple inputs
             if not self._has_inputs:
                 # Inferring model's inputs from data points for models with no explicit inputs
@@ -814,12 +879,12 @@ class IntegratedGradients(Explainer):
                 inputs = [tf.keras.Input(shape=xx.shape[1:], dtype=xx.dtype) for xx in X]
                 self.model(inputs, **forward_kwargs)
 
-            _validate_output(self.model, target)
+            _validate_output(self.model, target)  # type: ignore[arg-type]
 
             if self.layer is None:
                 # No layer passed, attributions computed with respect to the inputs
                 attributions = self._compute_attributions_list_input(X,
-                                                                     baselines,
+                                                                     baselines,  # type: ignore[arg-type]
                                                                      target,
                                                                      step_sizes,
                                                                      alphas,
@@ -830,10 +895,10 @@ class IntegratedGradients(Explainer):
             else:
                 # forwad inputs and  baselines
                 X_layer, baselines_layer = _forward_input_baseline(X,
-                                                                   baselines,
+                                                                   baselines,  # type: ignore[arg-type]
                                                                    self.model,
                                                                    self.layer,
-                                                                   self.orig_call,
+                                                                   self.orig_call,  # type: ignore[arg-type]
                                                                    forward_kwargs=forward_kwargs,
                                                                    forward_to_inputs=attribute_to_layer_inputs)
 
@@ -877,10 +942,10 @@ class IntegratedGradients(Explainer):
             else:
                 # forwad inputs and  baselines
                 X_layer, baselines_layer = _forward_input_baseline(X,
-                                                                   baselines,
+                                                                   baselines,  # type: ignore[arg-type]
                                                                    self.model,
                                                                    self.layer,
-                                                                   self.orig_call,
+                                                                   self.orig_call,  # type: ignore[arg-type]
                                                                    forward_kwargs=forward_kwargs,
                                                                    forward_to_inputs=attribute_to_layer_inputs)
 
@@ -906,34 +971,34 @@ class IntegratedGradients(Explainer):
         deltas = _compute_convergence_delta(self.model,
                                             input_dtypes,
                                             attributions,
-                                            baselines,
+                                            baselines,  # type: ignore[arg-type]
                                             X,
                                             forward_kwargs,
                                             target,
                                             self._is_list)
 
-        return self.build_explanation(
+        return self._build_explanation(
             X=X,
             forward_kwargs=forward_kwargs,
-            baselines=baselines,
+            baselines=baselines,  # type: ignore[arg-type]
             target=target,
             attributions=attributions,
             deltas=deltas
         )
 
-    def build_explanation(self,
-                          X: List[np.ndarray],
-                          forward_kwargs: Optional[dict],
-                          baselines: List[np.ndarray],
-                          target: Optional[List[int]],
-                          attributions: Union[List[np.ndarray], List[tf.Tensor]],
-                          deltas: np.ndarray) -> Explanation:
+    def _build_explanation(self,
+                           X: Union[List[np.ndarray], np.ndarray],
+                           forward_kwargs: Optional[dict],
+                           baselines: List[np.ndarray],
+                           target: Optional[List[int]],
+                           attributions: Union[List[np.ndarray], List[tf.Tensor]],
+                           deltas: np.ndarray) -> Explanation:
         if forward_kwargs is None:
             forward_kwargs = {}
         data = copy.deepcopy(DEFAULT_DATA_INTGRAD)
         predictions = self.model(X, **forward_kwargs).numpy()
         if isinstance(attributions[0], tf.Tensor):
-            attributions = [attr.numpy() for attr in attributions]
+            attributions = [attr.numpy() for attr in attributions]  # type: ignore[union-attr]
         data.update(X=X,
                     forward_kwargs=forward_kwargs,
                     baselines=baselines,
@@ -945,6 +1010,14 @@ class IntegratedGradients(Explainer):
         return Explanation(meta=copy.deepcopy(self.meta), data=data)
 
     def reset_predictor(self, predictor: Union[tf.keras.Model]) -> None:
+        """
+        Resets the predictor model.
+
+        Parameters
+        ----------
+        predictor
+            New prediction model.
+        """
         # TODO: check what else should be done (e.g. validate dtypes again?)
         self.model = predictor
 
@@ -971,18 +1044,18 @@ class IntegratedGradients(Explainer):
         step_sizes
             Weights in the path integral sum.
         alphas
-            Interpolation parameter defining the points of the interal path.
+            Interpolation parameter defining the points of the integral path.
         nb_samples
             Total number of samples.
         forward_kwargs
             Input keywords args.
         compute_layer_inputs_gradients
             In case of layers gradients, controls whether the gradients are computed for the layer's inputs or
-            outputs. If True, gradients are computed for the layer's inputs, if False for the layer's outputs.
+            outputs. If ``True``, gradients are computed for the layer's inputs, if ``False`` for the layer's outputs.
 
         Returns
         -------
-            Tuple with integrated gradients attributions, deltas and predictions
+        Tuple with integrated gradients attributions, deltas and predictions.
 
         """
         if forward_kwargs is None:
@@ -999,7 +1072,7 @@ class IntegratedGradients(Explainer):
 
         if forward_kwargs:
             paths_kwargs = {k: np.concatenate([forward_kwargs[k] for _ in range(self.n_steps)], axis=0)
-                            for k in forward_kwargs.keys()}
+                            for k in forward_kwargs.keys()}  # type: Optional[dict]
         else:
             paths_kwargs = None
 
@@ -1048,8 +1121,8 @@ class IntegratedGradients(Explainer):
             else:
                 grads_b = _gradients_layer(self.model,
                                            self.layer,
-                                           self.orig_call,
-                                           self.orig_dummy_input,
+                                           self.orig_call,  # type: ignore[arg-type]
+                                           self.orig_dummy_input,  # type: ignore[arg-type]
                                            paths_b,
                                            target_b,
                                            forward_kwargs=kwargs_b,
@@ -1095,18 +1168,18 @@ class IntegratedGradients(Explainer):
         step_sizes
             Weights in the path integral sum.
         alphas
-            Interpolation parameter defining the points of the interal path.
+            Interpolation parameter defining the points of the integral path.
         nb_samples
             Total number of samples.
         forward_kwargs
             Inputs keywords args.
         compute_layer_inputs_gradients
             In case of layers gradients, controls whether the gradients are computed for the layer's inputs or
-            outputs. If True, gradients are computed for the layer's inputs, if False for the layer's outputs.
+            outputs. If ``True``, gradients are computed for the layer's inputs, if ``False`` for the layer's outputs.
 
         Returns
         -------
-            Tuple with integrated gradients attributions, deltas and predictions
+        Tuple with integrated gradients attributions, deltas and predictions.
         """
         if forward_kwargs is None:
             forward_kwargs = {}
@@ -1115,7 +1188,7 @@ class IntegratedGradients(Explainer):
 
         if forward_kwargs:
             paths_kwargs = {k: np.concatenate([forward_kwargs[k] for _ in range(self.n_steps)], axis=0)
-                            for k in forward_kwargs.keys()}
+                            for k in forward_kwargs.keys()}  # type: Optional[dict]
         else:
             paths_kwargs = None
 
@@ -1162,8 +1235,8 @@ class IntegratedGradients(Explainer):
             else:
                 grads_b = _gradients_layer(self.model,
                                            self.layer,
-                                           self.orig_call,
-                                           self.orig_dummy_input,
+                                           self.orig_call,  # type: ignore[arg-type]
+                                           self.orig_dummy_input,  # type: ignore[arg-type]
                                            paths_b,
                                            target_b,
                                            forward_kwargs=kwargs_b,
